﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class GameController : MonoBehaviour
{
	public static bool GameStart = false;
	public static List<Vector2> EmptySpots = new List<Vector2>();

	public Text Player1ScoreText;
	public Text Player2ScoreText;
	public Text CountdownTimerText;
	public Text GameTimerText;
	public Text StartGameText;
	public Image FadeToBlackImage;
	public Text Player1WinsText;
	public Text Player2WinsText;
	public Text ItsADrawText;

	public static int Player1Score = 0;
	public static int Player2Score = 0;
	public float GameTimer = 60f;

	public GameObject ObstaclePrefab;

	private float Timer = 4f;
	private bool IsCountingDown = true;
	private float FadeRate = 0.01f;
	private bool HasStartedGame = false;
	private bool HasEndedGame = false;
	private float EndGameTimer = 5f;

	private const string GoText = "GO!";

	public static Bounds GridBounds = new Bounds();

	void Start()
	{
		float nodeSize = AstarPath.active.data.gridGraph.nodeSize;
		float width = AstarPath.active.data.gridGraph.Width * nodeSize;
		float height = AstarPath.active.data.gridGraph.Depth * nodeSize;
		GridBounds.size = new Vector3(width, height, 0f);

		Player1WinsText.enabled = false;
		Player2WinsText.enabled = false;
		ItsADrawText.enabled = false;
		Player1ScoreText.enabled = false;
		Player2ScoreText.enabled = false;
		CountdownTimerText.enabled = false;

		int gameTimerMinutes = (int)(GameTimer / 60f);
		int gameTimerSeconds = (int)(GameTimer - (gameTimerMinutes * 60f));
		GameTimerText.text = string.Format("{0:0}:{1:00}", gameTimerMinutes, gameTimerSeconds);
		GameTimerText.enabled = false;
	}

	void Update()
	{
		if (HasEndedGame == true)
		{
			if (FadeToBlackImage.color.a < 1f)
			{
				Color fadeToBlack = FadeToBlackImage.color;
				fadeToBlack.a += FadeRate;
				FadeToBlackImage.color = fadeToBlack;
			}
			else
			{
				if (Player1Score > Player2Score)
				{
					Player1WinsText.enabled = true;
				}
				else if (Player2Score > Player1Score)
				{
					Player2WinsText.enabled = true;
				}
				else
				{
					ItsADrawText.enabled = true;
				}

				if (EndGameTimer > 0)
				{
					EndGameTimer -= Time.deltaTime;
				}
				else
				{
					RestartGame();
				}
			}

			return;
		}

		if (HasStartedGame == false)
		{
			HasStartedGame = InputManager.Instance.GetCompDevInputButton(CompDevInput.Move, 0) || InputManager.Instance.GetCompDevInputButton(CompDevInput.Move, 1);
		}
		else if (IsCountingDown == true)
		{
			CountdownTimerText.enabled = true;
			StartGameText.enabled = false;

			if (Timer > 1f)
			{
				Timer -= Time.deltaTime;
				int countdown = (int)Mathf.Floor(Timer);
				CountdownTimerText.text = countdown.ToString();
			}
			else
			{
				Player1ScoreText.enabled = true;
				Player2ScoreText.enabled = true;
				GameTimerText.enabled = true;

				CountdownTimerText.text = GoText;
				IsCountingDown = false;
				GameStart = true;
				FindEmptySpots();
			}
		}
		else
		{
			if (CountdownTimerText.color.a > 0)
			{
				Color textColor = CountdownTimerText.color;
				textColor.a -= FadeRate;
				CountdownTimerText.color = textColor;
			}
			else
			{
				CountdownTimerText.enabled = false;
			}

			GameTimer -= Time.deltaTime;

			int gameTimerMinutes = (int)(GameTimer / 60f);
			int gameTimerSeconds = (int)(GameTimer - (gameTimerMinutes * 60f));

			GameTimerText.text = string.Format("{0:0}:{1:00}", gameTimerMinutes, gameTimerSeconds);

			Player1ScoreText.text = Player1Score.ToString();
			Player2ScoreText.text = Player2Score.ToString();

			if (GameTimer <= 0)
			{
				HasEndedGame = true;
			}
		}

		for (int i = 0; i < EmptySpots.Count; i++)
		{
			Vector2 point = EmptySpots[i];
			Vector3 start = new Vector3(point.x - 16, point.y, 0f);
			Vector3 end = new Vector3(point.x + 16, point.y, 0f);
			Debug.DrawLine(start, end, Color.red);
			Vector3 start1 = new Vector3(point.x, point.y -16, 0f);
			Vector3 end1 = new Vector3(point.x, point.y + 16, 0f);
			Debug.DrawLine(start1, end1, Color.red);
		}
	}

    public static Vector2 GetValidSpawnPoint() {
        bool invalidPosition = true;
		Vector2 spawnPosition = Vector2.zero;

		while (invalidPosition)
        {
			int randomIndex = Random.Range(0, EmptySpots.Count);
			Vector2 randomSpot = EmptySpots[randomIndex];
			spawnPosition.x = randomSpot.x;
			spawnPosition.y = randomSpot.y;

			RaycastHit2D hit = Physics2D.Raycast(spawnPosition, Vector2.zero);
            invalidPosition = hit.collider != null;
        }

        return spawnPosition;
    }

    private void FindEmptySpots()
    {
        EmptySpots.Clear();

        AstarPath Grid = AstarPath.active;

        float obstacleSize = ObstaclePrefab.GetComponent<SpriteRenderer>().bounds.size.x;

        float nodeSize = Grid.data.gridGraph.nodeSize;
        float width = Grid.data.gridGraph.width * nodeSize;
        float height = Grid.data.gridGraph.depth * nodeSize;

        float startX = -(width / 2f);
        float startY = -(height / 2f);

        startX += obstacleSize / 2f;
        startY += obstacleSize / 2f;

        for (float xPos = 0f; xPos < width; xPos += obstacleSize)
        {
            for (float yPos = 0f; yPos < height; yPos += obstacleSize)
            {
                float x = startX + xPos;
                float y = startY + yPos;

                Vector2 position = new Vector2(x, y);

                if (Physics2D.Raycast(position, Vector2.zero).collider == null)
                {
                    EmptySpots.Add(new Vector2(x, y));
                }
            }
        }
    }

    private void RestartGame()
    {
        GameStart = false;
        EmptySpots = new List<Vector2>();
        Player1ScoreText.enabled = false;
        Player2ScoreText.enabled = false;
        CountdownTimerText.enabled = false;
        GameTimerText.enabled = false;
        StartGameText.enabled = true;
        Color fadeToBlackColor = FadeToBlackImage.color;
        fadeToBlackColor.a = 0f;
        FadeToBlackImage.color = fadeToBlackColor;
        Player1WinsText.enabled = false;
        Player2WinsText.enabled = false;
        ItsADrawText.enabled = false;
        Player1Score = 0;
        Player2Score = 0;
        Timer = 4f;
        IsCountingDown = true;
        HasStartedGame = false;
        HasEndedGame = false;
        EndGameTimer = 5f;

        SceneManager.LoadScene(0);
    }
}
