﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Rigidbody2D))]
public class PackageCarrier : MonoBehaviour {

    protected Package HeldPackage = null;
    private PlayerNav Navigation = null;
    public int PlayerIndex;
	public AudioSource PackagePickupSound;
    public AudioSource PackageDropoffSound;
    public AudioSource StealSound;

    // Use this for initialization
    void Start () 
    {
        Navigation = gameObject.GetComponent<PlayerNav>();
        PlayerIndex = gameObject.GetComponent<PlayerMovement>().GamePadIndex;
    }

    void OnCollisionEnter2D(Collision2D other) {
        if (HeldPackage == null) {
            PackageCarrier otherCarrier = other.gameObject.GetComponent<PackageCarrier>();

            if (otherCarrier != null && otherCarrier.HoldingPackage()) {
                Vector2 myVelocity = GetComponent<Rigidbody2D>().velocity;
                Vector2 otherVelocity = otherCarrier.GetComponent<Rigidbody2D>().velocity;

                // Steal the other guy's package if our velocity is higher
                if (myVelocity.magnitude > otherVelocity.magnitude) {
                    Package stolenPackage = otherCarrier.ReleasePackage();
                    HoldPackage(stolenPackage);
                    if (StealSound.isPlaying == false)
					{
						StealSound.Play();
					}
                }
            }
        }
    }

    void OnTriggerEnter2D(Collider2D other) {
        if (HeldPackage == null)
        {
            Package packageToHold = other.gameObject.GetComponent<Package>();

            // If the package has a carrier, we'll check if we can steal it
            // based on our collision with the carrier
            bool packageIsFree = packageToHold != null && packageToHold.Carrier == null;
            if (HeldPackage == null && packageIsFree)
            {
                if (PackagePickupSound.isPlaying == false)
				{
					PackagePickupSound.Play();
				}
                HoldPackage(packageToHold);
            }
        }
        else
        {
            PackageTarget pTarget = other.gameObject.GetComponent<PackageTarget>();
            if (pTarget != null &&
                pTarget.ParentPackage == HeldPackage)
            {
                if (PackageDropoffSound.isPlaying == false)
				{
					PackageDropoffSound.Play();
				}

				HeldPackage.Delivered = true;
                
				Destroy(HeldPackage.PTarget.gameObject);
                Destroy(HeldPackage.gameObject);

                if (PlayerIndex == 0)
                {
                    GameController.Player1Score += HeldPackage.ScoreReward;
                }
                else if (PlayerIndex == 1)
                {
                    GameController.Player2Score += HeldPackage.ScoreReward;
                }

                Navigation.HasTarget = false;
                HeldPackage = null;
            }
        }
    }

    public bool HoldingPackage() {
        return HeldPackage != null;
    }

    // Start holding the given object
    void HoldPackage(Package packageToHold) {
        HeldPackage = packageToHold;
        packageToHold.transform.parent = transform;
        packageToHold.transform.localPosition = Vector3.zero;
        Vector2 target = packageToHold.Target;
        Navigation.Target = new Vector3(target.x, target.y, 0f);
        packageToHold.SpawnTarget();
        packageToHold.Carrier = gameObject;
    }

    // Release the package we're holding and return it.
    Package ReleasePackage() {
        Package releasedPackage = HeldPackage;
        Navigation.HasTarget = false;
        HeldPackage = null;
        return releasedPackage;
    }
}
