﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(PlayerMovement))]
 public class PowerupHolder : MonoBehaviour 
 {
    public GameObject PulsePrefab;
    public GameObject FreezeBeamPrefab;
    public GameObject FreezeBeamExplosionPrefab;
    // Number of seconds someone gets frozen when hit by a freeze beam
    public float FreezeDuration;
	public AudioSource FreezeSound;
    public AudioSource PulseSound;
    public AudioSource HazardDropSound;
    public AudioSource PowerupPickupSound;

	private const string POWERUP_TAG = "Powerup";

     protected Powerup HeldPowerup;

     protected int GamePadIndex;

     protected Dictionary<CompDevHazard, Object> hazardPrefabs;

     // Use this for initialization
     void Start () {
         HeldPowerup = null;

         GamePadIndex = GetComponent<PlayerMovement>().GamePadIndex;

         hazardPrefabs = new Dictionary<CompDevHazard, Object>();
         hazardPrefabs.Add(CompDevHazard.Stop, Resources.Load("StopHazard"));
         hazardPrefabs.Add(CompDevHazard.Boost, Resources.Load("BoostHazard"));
         hazardPrefabs.Add(CompDevHazard.Leech, Resources.Load("LeechHazard"));
         hazardPrefabs.Add(CompDevHazard.Teleport, Resources.Load("TeleportHazard"));
         hazardPrefabs.Add(CompDevHazard.Pulse, PulsePrefab);
     }

     public bool HoldingPowerup() {
         return HeldPowerup != null;
     }

     void Update() {
         bool usePowerupPressed = InputManager.Instance
             .GetCompDevInputButton(CompDevInput.UsePowerup, GamePadIndex);

         if (usePowerupPressed && HoldingPowerup()) {
             UsePowerup();
         }

     }

     void UsePowerup() {
         if (HeldPowerup.PowerupType != CompDevPowerup.None) {
         }
         switch (HeldPowerup.PowerupType) {
         case CompDevPowerup.None:
             break;
         case CompDevPowerup.Stop:
             Spawn(CompDevHazard.Stop);
             if (HazardDropSound.isPlaying == false)
             {
                 HazardDropSound.Play();
             }
             break;
         case CompDevPowerup.Boost:
             Spawn(CompDevHazard.Boost);
             if (HazardDropSound.isPlaying == false)
             {
                 HazardDropSound.Play();
             }
             break;
         case CompDevPowerup.Leech:
             Spawn(CompDevHazard.Leech);
             if (HazardDropSound.isPlaying == false)
			 {
				 HazardDropSound.Play();
			 }
             break;
         case CompDevPowerup.FreezeBeam:
             FireFreezeBeam();
             break;
         case CompDevPowerup.Pulse:
             Spawn(CompDevHazard.Pulse);
             if (PulseSound.isPlaying == false)
			 {
				 PulseSound.Play();
			 }
             break;
         case CompDevPowerup.Teleport:
             Spawn(CompDevHazard.Teleport);
             if (HazardDropSound.isPlaying == false)
			 {
				 HazardDropSound.Play();
			 }
             break;
         }

         Powerup powerup = HeldPowerup;
         HeldPowerup = null;
         Destroy(powerup.gameObject);
     }

     void Spawn(CompDevHazard hazardType) {
         Object prefab = hazardPrefabs[hazardType];
         GameObject spawnedHazard = (GameObject) Instantiate(
             prefab, transform.position, Quaternion.identity);

         spawnedHazard.GetComponent<Hazard>().Init(gameObject, hazardType);
     }

     void OnTriggerEnter2D(Collider2D other) {
         Powerup powerup = other.gameObject.GetComponent<Powerup>();

         bool canPickUp = !HoldingPowerup()
             && other.gameObject.tag == POWERUP_TAG
             && powerup != null
             && powerup.Holder == null;

         if (canPickUp) 
         {
            if (PowerupPickupSound.isPlaying == false)
            {
				PowerupPickupSound.Play();
			}

             HoldPowerup(powerup);

             if (GamePadIndex == 0) {
                 GameController.Player1Score += Powerup.PICKUP_POINT_VALUE;
             }
             else if (GamePadIndex == 1) {
                 GameController.Player2Score += Powerup.PICKUP_POINT_VALUE;
             }
         }
     }

     void HoldPowerup(Powerup powerupToHold) {
         HeldPowerup = powerupToHold;
         powerupToHold.transform.parent = transform;
         powerupToHold.transform.localPosition = Vector3.zero;

         powerupToHold.Holder = gameObject;
     }

     void FireFreezeBeam() {
         SpriteRenderer playerSprite = transform.Find("Sprite")
             .GetComponent<SpriteRenderer>();

         float playerWidth = playerSprite.bounds.size.x;
         Vector3 rightVector = Vector3.Cross(
             playerSprite.transform.right,
             -playerSprite.transform.forward);

         Vector2[] rayOrigins = {
             playerSprite.transform.position - (
                 rightVector * playerWidth/2),
             playerSprite.transform.position,
             playerSprite.transform.position + (
                 rightVector * playerWidth/2),
         };

         float rayAngle = playerSprite.transform.rotation.eulerAngles.z;
         Vector2 rayDirection = new Vector2(
             Mathf.Cos(Mathf.Deg2Rad * rayAngle),
             Mathf.Sin(Mathf.Deg2Rad * rayAngle));

         // Don't hit ourselves with our own beam!
         PlayerMovement thisPlayer = GetComponent<PlayerMovement>();

         RaycastHit2D firstHit = new RaycastHit2D();

         ContactFilter2D filter = new ContactFilter2D();
         filter.useTriggers = false;
         filter.useLayerMask = false;
         filter.useDepth = false;
         filter.useNormalAngle = false;

         for (int i = 0; i < rayOrigins.Length; ++i) {
             RaycastHit2D[] hits = new RaycastHit2D[2];
             int hitCount = Physics2D.Raycast(rayOrigins[i], rayDirection, filter, hits);
            
             if (hitCount > 0)
             {
                 RaycastHit2D hit = new RaycastHit2D();

                 for (int j = 0; j < hits.Length; j++)
                 {
                     RaycastHit2D possibleHit = hits[j];

                     if (possibleHit.collider != null)
                     {
                        PlayerMovement player = possibleHit.collider.GetComponent<PlayerMovement>();
						if (player != null && player == thisPlayer)
						{
						    continue;
						}

						hit = hits[j];
						break;
                     }
                 }

                 if (hit.collider != null)
                 {
                     if (hit.collider.isTrigger == false &&
                         firstHit.collider == null &&
                         i == 1)
                     {
                         firstHit = hit;
                     }
                     PlayerMovement player = hit.collider.GetComponent<PlayerMovement>();
                     if (player != null && player != thisPlayer)
                     {
                         player.Freeze(FreezeDuration);
                     }
                 }
             }
         }

         GameObject freezeBeam = Instantiate(FreezeBeamPrefab, transform.position, Quaternion.identity);
         LineRenderer freezeBeamLine = freezeBeam.GetComponent<LineRenderer>();
         freezeBeamLine.SetPosition(0, transform.position);

         if (firstHit.collider != null)
         {
             freezeBeamLine.SetPosition(1, firstHit.collider.transform.position);
         }
         else
         {
             Bounds gridBounds = GameController.GridBounds;
             float maxLength = Mathf.Max(gridBounds.size.x, gridBounds.size.y);
             Vector2 deltaPosition = rayDirection * maxLength;
             Vector3 position = transform.position;
             position.x += deltaPosition.x;
             position.y += deltaPosition.y;
             freezeBeamLine.SetPosition(1, position);
         }

		GameObject explosion = Instantiate(FreezeBeamExplosionPrefab, freezeBeamLine.GetPosition(1), Quaternion.identity);
		ParticleSystem explosionParticles = explosion.GetComponent<ParticleSystem>();
#pragma warning disable 618
		explosionParticles.startColor = Color.blue;
#pragma warning restore 618

        if (FreezeSound.isPlaying == false)
		{
			FreezeSound.Play();
		}
	}
 }
