﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Spawner))]
public class PowerupSpawner : MonoBehaviour {

    protected const string POWERUP_TAG = "Powerup";

    // Min time between a powerup being spawned
    public float MinSpawnInterval = 10.0f;

    // Max time between a powerup being spawned
    public float MaxSpawnInterval = 30.0f;

    // Max number of powerups spawned at a time
    public int MaxPowerups = 5;

    // Keep track of when we should spawn our next powerup
    protected float powerupSpawnCountdown;

    protected Dictionary<CompDevPowerup, Object> powerupPrefabs;

    // Use this for initialization
    void Start () {
        StartCoroutine(doCountdown());

        powerupPrefabs = new Dictionary<CompDevPowerup, Object>();
        powerupPrefabs.Add(CompDevPowerup.Stop, Resources.Load("StopPowerup"));
        powerupPrefabs.Add(CompDevPowerup.Boost, Resources.Load("BoostPowerup"));
        powerupPrefabs.Add(CompDevPowerup.Leech, Resources.Load("LeechPowerup"));
        powerupPrefabs.Add(CompDevPowerup.FreezeBeam, Resources.Load("FreezeBeamPowerup"));
        powerupPrefabs.Add(CompDevPowerup.Pulse, Resources.Load("PulsePowerup"));
        powerupPrefabs.Add(CompDevPowerup.Teleport, Resources.Load("TeleportPowerup"));
    }

    CompDevPowerup GetRandomPowerupType() {
        System.Array A = System.Enum.GetValues(typeof(CompDevPowerup));
        CompDevPowerup powerupType = CompDevPowerup.None;

        // We need to return a powerup type that isn't the default
        while (powerupType == CompDevPowerup.None) {
            powerupType = (CompDevPowerup) A.GetValue(Random.Range(0, A.Length));
        }
        return powerupType;
    }

    IEnumerator doCountdown() {
        for (;;) {
            float nextSpawnInterval = Random.Range(MinSpawnInterval, MaxSpawnInterval);

            yield return new WaitForSeconds(nextSpawnInterval);

            bool canSpawnPowerup = (GameObject.FindGameObjectsWithTag(POWERUP_TAG).Length < MaxPowerups);

            if (canSpawnPowerup) {
                Spawner spawner = GetComponent<Spawner>();
                Object spawnObject = powerupPrefabs[GetRandomPowerupType()];
                spawner.Spawn(spawnObject);
            }
        }
    }
}
